/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#ifndef TASKVIEWBUTTON_H
#define TASKVIEWBUTTON_H

#include <QToolButton>
#include <QMouseEvent>

#include "../panel/customstyle.h"
#include "../panel/iukuipanelplugin.h"
class TaskViewButton :public QToolButton
{
    Q_OBJECT
public:
    TaskViewButton(IUKUIPanelPlugin *plugin,QWidget *parent=nullptr);
    ~TaskViewButton();
    void realign();
protected:
    void mousePressEvent(QMouseEvent* event);

private:
    QWidget *m_parent;
    IUKUIPanelPlugin * m_plugin;
    QGSettings *m_gsettings;
    bool m_isDarkStyle;
private slots:
    void setSystemStyle();

};

#endif // TASKVIEWBUTTON_H
