/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#ifndef WATCHERMANAGER_H
#define WATCHERMANAGER_H

#include <QObject>
#include "filewatcher/filewatcher.h"

#include "dbus-server/server.h"
#include "dbus-server/dbus-adaptor.h"
#include "pin-totaskbar/pintotaskbar.h"
#include "pin-totaskbar/taskbar-dbus-adaptor.h"

class WatcherManager : public QObject
{
    Q_OBJECT
public:
    explicit WatcherManager(QObject *parent = nullptr);
    ~WatcherManager();
private:
    void register_dbus();

    PinToTaskbar* m_taskbarDBus;
};

#endif // WATCHERMANAGER_H
