/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU  Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#include "panel_commission.h"

PanelCommission::PanelCommission()
{
//    system("/usr/share/ukui/ukui-panel/ukui-panel-reset.sh");
}

PanelCommission::~PanelCommission()
{
}

void PanelCommission::panelConfigFileReset(bool reset)
{
    if(reset)
        system("/usr/share/ukui/ukui-panel/ukui-panel-reset.sh");
}

void PanelCommission::panelConfigFileValueInit(bool set)
{
    if(set)
        system("/usr/share/ukui/ukui-panel/panel-commission.sh");
}
