#!/bin/bash
## 定制版本的配置文件初始化处理

#判断文件是否存在
commissionFile="${HOME}/.config/ukui/panel-commission.ini"
if [[ ! -f "$commissionFile" ]]; then
  cp /usr/share/ukui/ukui-panel/panel-commission.ini ${HOME}/.config/ukui
fi


##拷贝配置文件到用户目录
panelConfigFile="${HOME}/.config/ukui/panel.conf"
echo $panelConfigFile
if [ ! -f "$panelConfigFile" ]; then
  cp /usr/share/ukui/panel.conf ${HOME}/.config/ukui
fi

